import json
import sys
import time
from datetime import datetime
from http.client import HTTPException
from pathlib import Path

from .database import insert_observation
from .fetcher import fetch_labels, fetch_live_data


def create_metadata_if_not_exists(
    database_path: str,
    live_data_url: str,
) -> dict[str, str]:
    path = Path(database_path).parent / (Path(database_path).stem + "_metadata.json")
    if not path.exists():
        try:
            labels = fetch_labels(live_data_url)
        except HTTPException as e:
            print(f"Error fetching metadata labels: {e}")
            return {}
        path.write_text(json.dumps(labels, indent=4))
        return labels
    return json.loads(path.read_text())


def clear_lines(n: int) -> None:
    for _ in range(n):
        print("\033[A\033[K", end="")


def wait_for_next_update(period_seconds: int) -> None:
    for i in range(period_seconds, 0, -1):
        print(f"Next update in {i} seconds", end="\r")
        time.sleep(1)


def start_daemon(
    live_data_url: str,
    database_path: str,
    period_seconds: int = 60,
) -> None:
    print(f"Observing {live_data_url}")
    print("Press Ctrl+C to stop")
    metadata = create_metadata_if_not_exists(database_path, live_data_url)
    remove_newlines = 0
    try:
        while True:
            clear_lines(remove_newlines)
            try:
                live_data = fetch_live_data(live_data_url)
            except HTTPException as e:
                print(f"Error fetching live data: {type(e).__name__}")
                remove_newlines = 2
                wait_for_next_update(period_seconds)
                continue
            print(f"Updated at: {datetime.now()}")
            labeled_data = {
                metadata.get(key, key): value for key, value in live_data.items()
            }
            pretty_data = json.dumps(labeled_data, indent=4)
            # + 2 for the "Updated at" line and the newline generated by print()
            remove_newlines = pretty_data.count("\n") + 2
            print(pretty_data)
            insert_observation(database_path, live_data)
            wait_for_next_update(period_seconds)
    except KeyboardInterrupt:
        print(f"\nStopping... results saved to {database_path}")
        sys.exit(0)
